-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD: Digital Circuits and Systems - DIGSYS         
--------------------------------------------------------------------------------
-- LAB7 example tutorial on prototyping: Counter_mod16_top
-- Target board: DE10-Lite Terasic 
-- Target PLD: MAX10 10M50DAF484C7

-- Plan C2: FSM and some additional components for:
-- Adapting buttons and LED
-- Generating CLK signal from board external oscillator
-- 7-segment decoding for displaying counter states as hexadecimal symbols

-- Reference and schematic at: 
-- https://digsys.upc.edu/csd/P07/Counter_mod16/Counter_mod16.html#prototyping
--------------------------------------------------------------------------------
LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;
			
ENTITY Counter_mod16_top IS
	Port (
		OSC_CLK_in  	: IN 	std_logic;
		CD_L			: IN	STD_LOGIC;
		CE, UD_L, LD	: IN	STD_LOGIC;   
		Sel_FREQ		: IN	STD_LOGIC; 		
		CLK_Source		: IN	STD_LOGIC; 			
		P_CLK_L			: IN	STD_LOGIC; 		
		Din				: IN	STD_LOGIC_VECTOR(3 DOWNTO 0);		
		HEX2_L      	: OUT	STD_LOGIC_VECTOR(6 DOWNTO 0);
		LED         	: OUT	STD_LOGIC_VECTOR(9 DOWNTO 0)		
   );

END Counter_mod16_top;

--------------------------------------------------------------------------------
	-- The description of the proposed schematic
	-- 
ARCHITECTURE schematic_planC2 OF Counter_mod16_top IS

	COMPONENT Counter_mod16 IS -- This is the counter to test in the board
      Port (	CLK 	  		: IN	STD_LOGIC;
				CD 	  			: IN	STD_LOGIC;
				LD, CE, UD_L	: IN	STD_LOGIC;			
				Din  			: IN	STD_LOGIC_VECTOR(3 DOWNTO 0);
				Q	  			: OUT	STD_LOGIC_VECTOR(3 DOWNTO 0);
				TC16  			: OUT	STD_LOGIC
          	);  
	END COMPONENT;

	COMPONENT CLK_Generator IS  
      PORT(	
			OSC_CLK_in		:	IN	STD_LOGIC;
			CD 				:	IN	STD_LOGIC;
			CE				:	IN	STD_LOGIC;
			CLK_16Hz_SQ	    :	OUT	STD_LOGIC; 
			CLK_1Hz_SQ	    :	OUT	STD_LOGIC
		);
	end COMPONENT;       
	
	COMPONENT HEX_7seg_DECODER IS  
	PORT (
		X_IN   								: IN STD_LOGIC_VECTOR(3 DOWNTO 0); 
		BI_L   								: IN STD_LOGIC;      
		A_L, B_L, C_L, D_L, E_L, F_L, G_L  	: OUT STD_LOGIC	
		);
	end COMPONENT; 

	COMPONENT MUX_2 IS  
		PORT (
			Ch0, Ch1	: 	IN 	STD_LOGIC; 
			S			:	IN 	STD_LOGIC;	
			E			: 	IN 	STD_LOGIC;
			Y			: 	OUT	STD_LOGIC
		  );
	end COMPONENT; 

	
-- Signals for connecting components together (the wires)
	SIGNAL CD, CLK, OSC_CLK, TC16, P_CLK	: std_logic;
	SIGNAL CLK_16Hz_SQ, CLK_1Hz_SQ			: std_logic;	
	SIGNAL Q								: STD_LOGIC_VECTOR(3 DOWNTO 0);
	
	BEGIN  
-- Instantiation of components
	Chip1 : Counter_mod16        
-- 	from component name => to signal or port name	
		PORT MAP (
			CLK			=> CLK,
			CD			=> CD,
			LD			=> LD,			
			CE			=> CE,
			UD_L		=> UD_L,
			Q			=> Q,
			TC16		=> TC16,
			Din			=> Din			
		);
   
	Chip2 : CLK_Generator        
-- 	from component name => to signal or port name	
		PORT MAP (
			CD				=> CD,
			CE				=> '1',
			OSC_CLK_in		=> OSC_CLK_in,
			CLK_16Hz_SQ	    => CLK_16Hz_SQ, 		
			CLK_1Hz_SQ	    => CLK_1Hz_SQ
		);       
		
	Chip3 : MUX_2
-- 	from component name => to signal or port name	
		PORT MAP (
			E			=> '1',
			S			=> Sel_FREQ,
			Ch0			=> CLK_1Hz_SQ, 
			Ch1	    	=> CLK_16Hz_SQ, 			
			Y           => OSC_CLK
		);  

	Chip4 : MUX_2
-- 	from component name => to signal or port name	
		PORT MAP (
			E			=> '1',
			S			=> CLK_Source,
			Ch0			=> P_CLK, 
			Ch1	    	=> OSC_CLK, 			
			Y           => CLK
		);  

		
	Chip5 : HEX_7seg_decoder        
-- 	from component name => to signal or port name	
	PORT MAP (
		X_IN			=> Q,			
		BI_L			=> '1',
		A_L				=> HEX2_L(0),		
		B_L				=> HEX2_L(1),	
		C_L				=> HEX2_L(2),	
		D_L				=> HEX2_L(3),	
		E_L				=> HEX2_L(4),	
		F_L				=> HEX2_L(5),	
		G_L				=> HEX2_L(6)		
	);          
  
-- Buffers and logic to adapt the board hardware
CD 				<= not (CD_L); 		-- The board has active-low push-buttons
P_CLK			<= not (P_CLK_L); -- This is the manual CLK using a KEY	
LED(3 downto 0) <= Q;
LED(9)			<= TC16;

-- All LED not used connected to '0': off
LED(8 downto 4) <= "00000";

    
END schematic_planC2;






